#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import os

abspath = os.path.abspath(__file__)
dname = os.path.dirname(abspath)
os.chdir(dname)

import argparse
from bottle.bottle import route, get, static_file, template, request, response, run, install
import logger

parser = argparse.ArgumentParser()
parser.add_argument("--config", dest="config", help='part to config.ini file, default is ../config.ini')
parser.set_defaults(config="../config.ini")
parser.add_argument("--port", type=int, dest="port", help='service listen port, default 14445')
parser.set_defaults(port=14445)

args = parser.parse_args()
configFile = args.config
port = args.port

logger.configFile = configFile
log = logger.getLogger("cs-state-service")

serviceLogsFiles = {
    "cs": "/opt/virgo/service.out",
    "cs_web": "/opt/virgo/service.out",
    "accounting": "/opt/virgo/logs/accounting/rpc-thrift.log",
    "accounting_rest": "/opt/virgo/logs/accounting/rpc-rest.log",
    "cs_sales_soap": "/opt/virgo/logs/cashserver-sales-ws-3.0.0/common.log",
    "cs_sales": "/opt/exchangers/logs/saleUnloadmain.log",
    "cs_private_office_service": "/opt/virgo/logs/private-office-service-3.0.0/common.log",
    "data_transfer": "/opt/virgo/logs/data-transfer/unknown/schedule.log",
    "exchange_file": "/opt/exchangers/logs/dictLoadmain.log",
    "exchange_soap": "/opt/exchangers/logs/dictLoadsoap.log",
    "exchange_rest": "/opt/exchangers/logs/dictLoadrest.log",
    "nes": "/opt/nes/logs/nesUploadService.log",
    "qpid": "/var/log/qpid/qpid.log",
    "mysql": "/var/log/mysql/mysql.log",
    "mongo": "/var/log/mongodb/mongodb.log"
}


# Root page
@get('/')
def indexPage():
    return stylesheets('index')


@get('/favicon.ico')
def getFavicon():
    return static_file('favicon.ico', root='images')


# Static Routes
@get('/js/<filename:re:.*\.js>')
def javascripts(filename):
    return static_file(filename, root='js')


@get('/css/<filename:re:.*\.css>')
def stylesheets(filename):
    return static_file(filename, root='css')


@get('/fonts/<filename:re:.*\.(eot|svg|ttf|woff|woff2)>')
def stylesheets(filename):
    return static_file(filename, root='fonts')


@get('/<filename:re:index\.(html|htm)>')
def stylesheets(filename):
    return template('templates/index.tpl', state=getCurrentServicesState())


@route('/v1/state')
def getCurrentServicesState():
    host = request.urlparts.hostname
    return {
        "success": True,
        "services": [
            {
                "id": "cs",
                "name": u"Кассовый сервер",
                "isRunning":
                    os.system(
                        "test \"`curl --connect-timeout 1 -s -o /dev/null -w '%{http_code}' 'http://localhost:8080/CSrest/rest/system/info' 2>/dev/null`\" = \"200\"") == 0,
                "inAutostart": os.path.exists("/opt/virgo/pickup/cashserver-unlimited.plan"),
                "logFile": serviceLogsFiles["cs"],
                "logFileText": u"лог запуска",
                "goButtonLink": "http://%s:8080" % host
            },
            {
                "id": "cs_web",
                "name": u"Web кассового сервера",
                "isRunning":
                    os.system(
                        "test \"`curl --connect-timeout 1 -s -o '/dev/null' -w '%{http_code}' 'http://localhost:8080/CS/CashServerWeb/login'`\" = \"200\"") == 0,
                "inAutostart": os.path.exists("/opt/virgo/pickup/cashserver-web.plan"),
                "logFile": serviceLogsFiles["cs_web"],
                "logFileText": u"лог запуска",
                "goButtonLink": "http://%s:8080" % host
            },
            {
                "id": "cs_sales_soap",
                "name": u"Soap-продажи",
                "isRunning":
                    os.system(
                        'bash -c \'[[ "200 401 403" =~ "`curl --connect-timeout 1 -s -o /dev/null -w \'%{http_code}\' \'http://localhost:8080/sales-ws/sales.wsdl\' 2>/dev/null`" ]]\''
                    ) == 0,
                "inAutostart": os.path.exists("/opt/virgo/pickup/sales-soap-service.plan"),
                "logFile": serviceLogsFiles["cs_sales"],
                "goButtonText": u"перейти на wsdl",
                "goButtonLink": "http://%s:8080/sales-ws/sales.wsdl" % host
            },
            {
                "id": "cs_sales",
                "name": u"Продажи через файлы (exchanger-unload)",
                "isRunning": os.system("status exchangers-unload 2>/dev/null|grep -q 'start/running'") == 0,
                "inAutostart": os.path.exists("/etc/default/exchangers") and os.system("grep -q 'UNLOAD_START\s*=\s*yes' /etc/default/exchangers") == 0,
                "logFile": serviceLogsFiles["cs_sales"],
            },
            {
                "id": "cs_private_office_service",
                "name": u"Soap-API бонусного сервера (private-office-service)",
                "isRunning":
                    os.system(
                        'bash -c \'[[ "200 401 403" =~ "`curl --connect-timeout 1 -s -o /dev/null -w \'%{http_code}\' \'http://localhost:8080/private-office-service/servlet/AdminServlet\' 2>/dev/null`" ]]\''
                    ) == 0,
                "inAutostart": os.path.exists("/opt/virgo/pickup/private-office-service.plan"),
                "logFile": serviceLogsFiles["cs_private_office_service"],
                "goButtonText": u"перейти на wsdl",
                "goButtonLink": "http://%s:8080/private-office-service/services/PrivateOffice?wsdl" % host
            },
            {
                "id": "accounting",
                "name": u"Accounting-сервер",
                "isRunning": os.system("nc -q 1 -w2 -z localhost 7791") == 0,
                "inAutostart": os.path.exists("/opt/virgo/pickup/accounting.plan"),
                "logFile": serviceLogsFiles["accounting"],
            },
            {
                "id": "accounting_rest",
                "name": u"Rest-API Accounting-сервера",
                "isRunning":
                    os.system(
                        "test \"`curl --connect-timeout 1 -s -o /dev/null -w '%{http_code}' 'http://localhost:8080/ACC/rest/v1/version' 2>/dev/null`\" = \"200\"") == 0,
                "inAutostart": os.path.exists("/opt/virgo/pickup/accounting_rest.plan"),
                "logFile": serviceLogsFiles["accounting_rest"],
                "goButtonLink": "http://%s:8080/ACC/rest/v1/version" % host
            },
            {
                "id": "data_transfer",
                "name": u"Data-transfer обмен",
                "isRunning":
                    os.path.exists("/opt/virgo/pickup/cashserver-datatransfer.plan") and
                    os.system("grep \"'cashserver-datatransfer-plan'\" /opt/virgo/service.out|tail -n1|grep -q \"Started plan\"") == 0,
                "inAutostart": os.path.exists("/opt/virgo/pickup/cashserver-datatransfer.plan"),
                "logFile": serviceLogsFiles["data_transfer"]
            },
            {
                "id": "exchange_file",
                "name": u"Файловый обмен (exchanger)",
                "isRunning": os.system("status exchangers-upload 2>/dev/null|grep -q 'start/running'") == 0,
                "inAutostart": os.path.exists("/etc/default/exchangers") and os.system("grep -q 'UPLOAD_START\s*=\s*yes' /etc/default/exchangers") == 0,
                "logFile": serviceLogsFiles["exchange_file"],
            },
            {
                "id": "exchange_soap",
                "name": u"Soap-обмен (exchanger-soap)",
                "isRunning": os.system("nc -q 1 -w2 -z localhost 18080") == 0,
                "inAutostart": os.path.exists("/etc/default/exchangers") and os.system("grep -q 'UPLOAD_SOAP_START\s*=\s*yes' /etc/default/exchangers") == 0,
                "logFile": serviceLogsFiles["exchange_soap"],
                "goButtonText": u"перейти на wsdl",
                "goButtonLink": "http://%s:18080/?wsdl" % host
            },
            {
                "id": "exchange_rest",
                "name": u"Rest-обмен (exchanger-rest)",
                "isRunning": os.system("nc -q 1 -w2 -z localhost 28080") == 0,
                "inAutostart": os.path.exists("/etc/default/exchangers") and os.system("grep -q 'UPLOAD_REST_START\s*=\s*yes' /etc/default/exchangers") == 0,
                "logFile": serviceLogsFiles["exchange_rest"],
            },
            {
                "id": "nes",
                "name": u"nes",
                "isRunning": os.system("status nes 2>/dev/null|grep -q 'start/running'") == 0,
                "inAutostart": os.path.exists("/etc/init/nes.conf"),
                "logFile": serviceLogsFiles["exchange_rest"],
            },
            {
                "id": "qpid",
                "name": u"Qpid",
                "isRunning": os.system("nc -q 1 -w2 -z localhost 5672") == 0,
                "inAutostart": os.path.exists("/etc/init.d/qpidd"),
                "logFile": serviceLogsFiles["qpid"]
            },
            {
                "id": "mysql",
                "name": u"Mysql",
                "isRunning": os.system("nc -q 1 -w2 -z localhost 3306") == 0,
                "inAutostart": os.path.exists("/etc/init/mysql.conf"),
                "logFile": serviceLogsFiles["mysql"]
            },
            {
                "id": "mongo",
                "name": u"MongoDB",
                "isRunning": os.system("nc -q 1 -w2 -z localhost 27017") == 0,
                "inAutostart": os.path.exists("/etc/init/mongodb.conf") or os.path.exists("/etc/init/mongod.conf"),
                "logFile": serviceLogsFiles["mongo"]
            }
        ]
    }


@route('/v1/logs/<service>')
def getLog(service):
    filename = serviceLogsFiles.get(service)
    if not filename:
        return "Not found"
    if not os.path.exists(filename):
        return "File with logs (%s) not found" % filename
    return template("templates/logs.tpl", service=service, filename=filename)


@route('/v1/logs/<service>/size')
def getLogSize(service):
    filename = serviceLogsFiles.get(service)
    if not filename:
        return 0
    return {
        "service": service,
        "filename": filename,
        "size": os.path.getsize(filename)
    }


@route('/v1/logs/<service>/part/begin/<beginIndex:int>/end/<endIndex:int>')
def getLogServicePart(service, beginIndex, endIndex):
    filename = serviceLogsFiles.get(service)
    if not filename:
        return None
    with open(filename) as fin:
        fin.seek(beginIndex)
        data = fin.read(endIndex - beginIndex)
    return data


# bottle access to log file plugin
def bootle_logger(func):
    def wrapper(*args, **kwargs):
        log.debug('%s %s %s %s' % (request.remote_addr, request.method, request.url, response.status))
        req = func(*args, **kwargs)
        return req

    return wrapper


if __name__ == '__main__':
    install(bootle_logger)
    run(server="paste", host="0.0.0.0", port=port)
